@@ -1,5 +1,7 @@
 /* Route flap dampening
- * Copyright (C) 2000 Kunihiro Ishiguro
+ * Copyright (C) 2000 DML Networks, Inc.
+ *
+ * Written by Kunihiro Ishiguro <kunihiro@zebra.org>
  *
  * This file is part of GNU Zebra.
  *
@@ -19,3 +21,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
+struct flap
+{
+  
+};
+
+struct flap_configuration
+{
+  ;
+};
+
