@@ -109,6 +109,6 @@ int zlog_rotate ();
 char *lookup (struct message *, int);
 char *mes_lookup (struct message *meslist, int max, int index);
 
-const char *zlog_priority [0]  ;
+extern const char *zlog_priority [0]  ;
 
 #endif /* _ZEBRA_LOG_H */
