@@ -23,7 +23,7 @@
 
 #include "newlist.h"
 #include "memory.h"
-
+
 struct newlist *
 newlist_new ()
 {
@@ -55,7 +55,7 @@ newnode_free (struct newnode *newnode)
 {
   XFREE (MTYPE_NEWNODE, newnode);
 }
-
+
 void
 newnode_add (struct newlist *list, void *val)
 {
@@ -135,11 +135,3 @@ newlist_delete (struct newlist *list)
     }
   newlist_free (list);
 }
-
-void *
-newlist_first (struct newlist *list)
-{
-  if (list->head)
-    return list->head->data;
-  return NULL;
-}
