@@ -78,10 +78,6 @@
 #include <stdarg.h>
 #include <assert.h>
 
-#ifdef HAVE_PTHREAD_H
-#include <pthread.h>
-#endif /* HAVE_PTHREAD_H */
-
 /* network include group */
 
 #include <sys/socket.h>
@@ -227,6 +223,7 @@
 #define ZEBRA_FLAG_INTERNAL           0x01
 #define ZEBRA_FLAG_SELFROUTE          0x02
 #define ZEBRA_FLAG_BLACKHOLE          0x04
+#define ZEBRA_FLAG_IBGP               0x08
 
 #ifndef INADDR_LOOPBACK
 #define	INADDR_LOOPBACK	0x7f000001	/* Internet address 127.0.0.1.  */
@@ -268,4 +265,11 @@ typedef u_char safi_t;
 typedef u_int16_t zebra_size_t;
 typedef u_int8_t zebra_command_t;
 
+#ifdef HAVE_REPAIRABLE_HTONL
+#define htonl(x) __cpu_to_be32(x)
+#define ntohl(x) __be32_to_cpu(x)
+#define htons(x) __cpu_to_be16(x)
+#define ntohs(x) __be16_to_cpu(x)
+#endif
+
 #endif /* _ZEBRA_H */
