@@ -53,7 +53,7 @@ struct ospf6_route_node_info
 {
   unsigned char     dest_type;       /* Destination Type */
   unsigned char     opt_cap[3];      /* Optional Capability */
-  struct area      *area;            /* Associated area */
+  struct ospf6_area *area;            /* Associated area */
     /* note: multiple entry for the same ABR is different
        in routing table by index */
   unsigned char     path_type;       /* Path-type */
@@ -99,12 +99,12 @@ unsigned long ospf6_route_get_dst_rtrid (struct prefix_ipv6 *);
 unsigned long ospf6_route_get_dst_ifid (struct prefix_ipv6 *);
 
 char *ospf6_route_str (struct route_node *, char *, size_t);
-void ospf6_route_vty (struct vty *, struct route_node *);
+void ospf6_route_vty (struct vty *, struct route_node *, int);
 
 int ospf6_route_calc (struct thread *);
+int ospf6_route_calculation (struct thread *);
 void ospf6_route_update_zebra ();
 
-void ospf6_route_vty_new (struct vty *, struct route_node *, int);
 
 void ospf6_rtable_init ();
 
