@@ -63,7 +63,7 @@ ospf_find_abr_route (struct route_table *rtrs,
 
   for (node = listhead ((list) rn->info); node; nextnode (node))
     if ((or = getdata (node)) != NULL)
-      if (or->u.std.area == area && (or->u.std.flags & ROUTER_LSA_BORDER))
+      if (IPV4_ADDR_SAME (&or->u.std.area_id, &area->area_id) && (or->u.std.flags & ROUTER_LSA_BORDER))
 	return or;
 
   return NULL;
@@ -76,7 +76,7 @@ ospf_ia_network_route (struct route_table *rt, struct prefix_ipv4 *p,
   struct route_node *rn1;
   struct ospf_route *or;
 
-  zlog_info ("Z: ospf_ia_network_route(): processing summary route to %s/%d", 
+  zlog_info ("ospf_ia_network_route(): processing summary route to %s/%d", 
              inet_ntoa (p->prefix), p->prefixlen);
 
   /* Find a route to the same dest */
@@ -88,9 +88,8 @@ ospf_ia_network_route (struct route_table *rt, struct prefix_ipv4 *p,
 
       if ((or = rn1->info))
 	{
-	  zlog_info ("Z: ospf_ia_network_route(): "
+	  zlog_info ("ospf_ia_network_route(): "
 		     "Found a route to the same network");
-
 	  /* Check the existing route. */
 	  if ((res = ospf_route_cmp (new_or, or)) < 0)
 	    {
@@ -117,9 +116,8 @@ ospf_ia_network_route (struct route_table *rt, struct prefix_ipv4 *p,
     } /*if (rn1)*/
   else
     { /* no route */
-      zlog_info ("Z: ospf_ia_network_route(): add new route to %s/%d",
+      zlog_info ("ospf_ia_network_route(): add new route to %s/%d",
                  inet_ntoa (p->prefix), p->prefixlen);
-
       ospf_route_add (rt, p, new_or, abr_or);
     }
 }
@@ -132,9 +130,8 @@ ospf_ia_router_route (struct route_table *rt, struct prefix_ipv4 *p,
   struct ospf_route *or = NULL;
   int ret;
 
-  zlog_info ("Z: ospf_ia_router_route(): considering %s/%d", 
+  zlog_info ("ospf_ia_router_route(): considering %s/%d", 
              inet_ntoa (p->prefix), p->prefixlen);
-
   /* Find a route to the same dest */
   rn = route_node_get (rt,(struct prefix *) p);
    
@@ -143,16 +140,18 @@ ospf_ia_router_route (struct route_table *rt, struct prefix_ipv4 *p,
     rn->info = list_init ();
   else
     {
+      struct ospf_area *or_area;
+      or_area = ospf_area_lookup_by_area_id (new_or->u.std.area_id);
+      assert (or_area);
       /* This is an additional route */
       route_unlock_node (rn);
-      or = ospf_find_asbr_route_through_area (rt, p, new_or->u.std.area);
+      or = ospf_find_asbr_route_through_area (rt, p, or_area);
     }
 
   if (or)
     {
-      zlog_info ("Z: ospf_ia_router_route(): "
+      zlog_info ("ospf_ia_router_route(): "
                  "a route to the same ABR through the same area exists");
-
       /* New route is better */
       if ((ret = ospf_route_cmp (new_or, or)) < 0)
 	{
@@ -163,7 +162,7 @@ ospf_ia_router_route (struct route_table *rt, struct prefix_ipv4 *p,
       /* Routes are the same */
       else if (ret == 0)
 	{
-	  zlog_info ("Z: ospf_ia_router_route(): merging the new route");
+	  zlog_info ("ospf_ia_router_route(): merging the new route");
 
 	  ospf_route_copy_nexthops (or, abr_or->path);
 	  ospf_route_free (new_or);
@@ -172,8 +171,7 @@ ospf_ia_router_route (struct route_table *rt, struct prefix_ipv4 *p,
       /* New route is worse */
       else
 	{
-	  zlog_info ("Z: ospf_ia_router_route(): skipping the new route");
-
+	  zlog_info ("ospf_ia_router_route(): skipping the new route");
 	  ospf_route_free (new_or);
 	  return;
 	}
@@ -181,7 +179,8 @@ ospf_ia_router_route (struct route_table *rt, struct prefix_ipv4 *p,
 
   ospf_route_copy_nexthops (new_or, abr_or->path);
 
-  zlog_info ("Z: ospf_ia_router_route(): adding the new route"); 
+  zlog_info ("ospf_ia_router_route(): adding the new route"); 
+
   list_add_node (rn->info, new_or);
 }
 
@@ -209,7 +208,7 @@ process_summary_lsa (struct ospf_lsa *l, void *v, int i)
   args = (struct ia_args *) v;
   sl = (struct summary_lsa *) l->data;
 
-  zlog_info ("Z: process_summary_lsa(): LS ID: %s", inet_ntoa (sl->header.id));
+  zlog_info ("process_summary_lsa(): LS ID: %s", inet_ntoa (sl->header.id));
 
   metric = GET_METRIC (sl->metric);
    
@@ -260,7 +259,7 @@ process_summary_lsa (struct ospf_lsa *l, void *v, int i)
   new_or->u.std.options = sl->header.options;
   new_or->u.std.origin = (struct lsa_header *) sl;
   new_or->cost = abr_or->cost + metric;
-  new_or->u.std.area = args->area;
+  new_or->u.std.area_id = args->area->area_id;
   new_or->path_type = OSPF_PATH_INTER_AREA;
 
   if (sl->header.type == OSPF_SUMMARY_LSA)
@@ -315,7 +314,7 @@ ospf_update_network_route (struct route_table *rt,
 
   if (abr_or == NULL)
     {
-      zlog_info ("Z: ospf_update_network_route(): can't find a route to the ABR");
+      zlog_info ("ospf_update_network_route(): can't find a route to the ABR");
       return;
     }
 
@@ -328,10 +327,8 @@ ospf_update_network_route (struct route_table *rt,
       if (ospf_top->abr_type != OSPF_ABR_SHORTCUT)
         return; /* Standard ABR can update only already installed
                    backbone paths                                       */
-
-      zlog_info ("Z: ospf_update_network_route(): "
+      zlog_info ("ospf_update_network_route(): "
                  "Allowing Shortcut ABR to add new route");
-
       new_or = ospf_route_new ();
       new_or->type = OSPF_DESTINATION_NETWORK;
       new_or->id = lsa->header.id;
@@ -339,7 +336,7 @@ ospf_update_network_route (struct route_table *rt,
       new_or->u.std.options = lsa->header.options;
       new_or->u.std.origin = (struct lsa_header *) lsa;
       new_or->cost = cost;
-      new_or->u.std.area = area;
+      new_or->u.std.area_id = area->area_id;
       new_or->path_type = OSPF_PATH_INTER_AREA;
       ospf_route_add (rt, p, new_or, abr_or);
 
@@ -357,25 +354,25 @@ ospf_update_network_route (struct route_table *rt,
   if (or->path_type != OSPF_PATH_INTRA_AREA &&
       or->path_type != OSPF_PATH_INTER_AREA)
     {
-      zlog_info ("Z: ospf_update_network_route(): ERR: path type is wrong");
+      zlog_info ("ospf_update_network_route(): ERR: path type is wrong");
       return;
     }
 
   if (ospf_top->abr_type == OSPF_ABR_SHORTCUT)
     {
       if (or->path_type == OSPF_PATH_INTRA_AREA &&
-	  !OSPF_IS_AREA_BACKBONE (or->u.std.area))
+	  !OSPF_IS_AREA_ID_BACKBONE (or->u.std.area_id))
 	{
-	  zlog_info ("Z: ospf_update_network_route(): Shortcut: "
+	  zlog_info ("ospf_update_network_route(): Shortcut: "
 		     "this intra-area path is not backbone");
 	  return;
 	}
     }
   else   /* Not Shortcut ABR */
     {
-      if (!OSPF_IS_AREA_BACKBONE (or->u.std.area))
+      if (!OSPF_IS_AREA_ID_BACKBONE (or->u.std.area_id))
 	{
-	  zlog_info ("Z: ospf_update_network_route(): "
+	  zlog_info ("ospf_update_network_route(): "
 		     "route is not BB-associated");
 	  return; /* We can update only BB routes */
 	}
@@ -383,29 +380,29 @@ ospf_update_network_route (struct route_table *rt,
 
   if (or->cost < cost)
     {
-      zlog_info ("Z: ospf_update_network_route(): new route is worse");
+      zlog_info ("ospf_update_network_route(): new route is worse");
       return;
     }
 
   if (or->cost == cost)
     {
-      zlog_info ("Z: ospf_update_network_route(): "
+      zlog_info ("ospf_update_network_route(): "
                  "new route is same distance, adding nexthops");
       ospf_route_copy_nexthops (or, abr_or->path);
     }
 
   if (or->cost > cost)
     {
-      zlog_info ("Z: ospf_update_network_route(): "
+      zlog_info ("ospf_update_network_route(): "
                  "new route is better, overriding nexthops");
       ospf_route_subst_nexthops (or, abr_or->path);
       or->cost = cost;
 
       if ((ospf_top->abr_type == OSPF_ABR_SHORTCUT) &&
-	  !OSPF_IS_AREA_BACKBONE (or->u.std.area))
+	  !OSPF_IS_AREA_ID_BACKBONE (or->u.std.area_id))
 	{
 	  or->path_type = OSPF_PATH_INTER_AREA;
-	  or->u.std.area = area;
+	  or->u.std.area_id = area->area_id;
 
           /* Note that we can do this only in Shortcut ABR mode,
              because standard ABR must leave the route type and area
@@ -434,7 +431,7 @@ ospf_update_router_route (struct route_table *rtrs,
 
   if (abr_or == NULL)
     {
-      zlog_info ("Z: ospf_update_router_route(): can't find a route to the ABR");
+      zlog_info ("ospf_update_router_route(): can't find a route to the ABR");
       return;
     }
 
@@ -472,7 +469,7 @@ ospf_update_router_route (struct route_table *rtrs,
 	  new_or->u.std.options = lsa->header.options;
 	  new_or->u.std.origin = (struct lsa_header *)lsa;
 	  new_or->cost = cost;
-	  new_or->u.std.area = area;
+	  new_or->u.std.area_id = area->area_id;
 	  new_or->path_type = OSPF_PATH_INTER_AREA;
 	  new_or->u.std.flags = ROUTER_LSA_EXTERNAL;
 	  ospf_ia_router_route (rtrs, p, new_or, abr_or);
@@ -485,7 +482,7 @@ ospf_update_router_route (struct route_table *rtrs,
 
   if (!(or->u.std.flags & ROUTER_LSA_EXTERNAL))
     {
-      zlog_info ("Z: ospf_upd_router_route(): the remote router is not an ASBR");
+      zlog_info ("ospf_upd_router_route(): the remote router is not an ASBR");
       return;
     }
 
@@ -524,26 +521,25 @@ process_transit_summary_lsa (struct ospf_lsa *l, void *v, int i)
   args = (struct ia_args *) v;
   sl = (struct summary_lsa *) l->data;
 
-  zlog_info ("Z: process_transit_summaries(): LS ID: %s",
+  zlog_info ("process_transit_summaries(): LS ID: %s",
              inet_ntoa (l->data->id));
-
   metric = GET_METRIC (sl->metric);
    
   if (metric == OSPF_LS_INFINITY)
     {
-      zlog_info ("Z: process_transit_summaries(): metric is infinity, skip");
+      zlog_info ("process_transit_summaries(): metric is infinity, skip");
       return 0;
     }
 
   if (IS_LSA_MAXAGE (l))
     {
-      zlog_info ("Z: process_transit_summaries(): This LSA is too old");
+      zlog_info ("process_transit_summaries(): This LSA is too old");
       return 0;
     }
 
   if (ospf_lsa_is_self_originated (l))
     { 
-      zlog_info ("Z: process_transit_summaries(): This LSA is mine, skip");
+      zlog_info ("process_transit_summaries(): This LSA is mine, skip");
       return 0;
     }
 
@@ -584,7 +580,7 @@ ospf_ia_routing (struct route_table *rt,
 {
   struct ospf_area * area;
 
-  zlog_info ("Z: ospf_ia_routing():start");
+  zlog_info ("ospf_ia_routing():start");
 
   if (OSPF_IS_ABR)
     {
@@ -594,40 +590,40 @@ ospf_ia_routing (struct route_table *rt,
       switch (ospf_top->abr_type)
         {
         case OSPF_ABR_STAND:
-          zlog_info ("Z: ospf_ia_routing():Standard ABR");
+          zlog_info ("ospf_ia_routing():Standard ABR");
 
           if ((area = ospf_top->backbone))
             {
               listnode node;
 
-              zlog_info ("Z: ospf_ia_routing():backbone area found");
-              zlog_info ("Z: ospf_ia_routing():examining summaries");
+              zlog_info ("ospf_ia_routing():backbone area found");
+              zlog_info ("ospf_ia_routing():examining summaries");
+
               OSPF_EXAMINE_SUMMARIES_ALL (area, rt, rtrs);
 
-              LIST_ITERATOR (ospf_top->areas, node)
+	      for (node = listhead (ospf_top->areas); node; nextnode (node))
                 if ((area = getdata (node)) != NULL)
                   if (area != ospf_top->backbone)
 		    if (ospf_area_is_transit (area))
 		      OSPF_EXAMINE_TRANSIT_SUMMARIES_ALL (area, rt, rtrs);
             }
           else
-            zlog_info ("Z: ospf_ia_routing():backbone area NOT found");
+            zlog_info ("ospf_ia_routing():backbone area NOT found");
           break;
         case OSPF_ABR_IBM:
         case OSPF_ABR_CISCO:
-          zlog_info ("Z: ospf_ia_routing():Alternative Cisco/IBM ABR");
-
+          zlog_info ("ospf_ia_routing():Alternative Cisco/IBM ABR");
           area = ospf_top->backbone; /* Find the BB */
 
           /* If we have an active BB connection */
           if (area && ospf_act_bb_connection ())
             {
-              zlog_info ("Z: ospf_ia_routing(): backbone area found");
-              zlog_info ("Z: ospf_ia_routing(): examining BB summaries");
+              zlog_info ("ospf_ia_routing(): backbone area found");
+              zlog_info ("ospf_ia_routing(): examining BB summaries");
 
               OSPF_EXAMINE_SUMMARIES_ALL (area, rt, rtrs);
 
-              LIST_ITERATOR (ospf_top->areas, node)
+	      for (node = listhead (ospf_top->areas); node; nextnode (node))
                 if ((area = getdata (node)) != NULL)
                   if (area != ospf_top->backbone)
 		    if (ospf_area_is_transit (area))
@@ -635,28 +631,26 @@ ospf_ia_routing (struct route_table *rt,
             }
           else
             { /* No active BB connection--consider all areas */
-              zlog_info ("Z: ospf_ia_routing(): "
+              zlog_info ("ospf_ia_routing(): "
                          "Active BB connection not found");
-
               LIST_ITERATOR (ospf_top->areas, node)
                 if ((area = getdata (node)) != NULL)
                   OSPF_EXAMINE_SUMMARIES_ALL (area, rt, rtrs);
             }
           break;
         case OSPF_ABR_SHORTCUT:
-          zlog_info ("Z: ospf_ia_routing():Alternative Shortcut");
-
+          zlog_info ("ospf_ia_routing():Alternative Shortcut");
           area = ospf_top->backbone; /* Find the BB */
 
           /* If we have an active BB connection */
           if (area && ospf_act_bb_connection ())
             {
-              zlog_info ("Z: ospf_ia_routing(): backbone area found");
-              zlog_info ("Z: ospf_ia_routing(): examining BB summaries");
+              zlog_info ("ospf_ia_routing(): backbone area found");
+              zlog_info ("ospf_ia_routing(): examining BB summaries");
               OSPF_EXAMINE_SUMMARIES_ALL (area, rt, rtrs);
             }
 
-          LIST_ITERATOR (ospf_top->areas, node)
+	  for (node = listhead (ospf_top->areas); node; nextnode (node))
             if ((area = getdata (node)) != NULL)
               if (area != ospf_top->backbone)
 		if (ospf_area_is_transit (area) ||
@@ -674,9 +668,9 @@ ospf_ia_routing (struct route_table *rt,
     {
       listnode node;
 
-      zlog_info ("Z: ospf_ia_routing():not ABR, considering all areas");
+      zlog_info ("ospf_ia_routing():not ABR, considering all areas");
 
-      LIST_ITERATOR (ospf_top->areas, node)
+      for (node = listhead (ospf_top->areas); node; nextnode (node))
         if ((area = getdata (node)) != NULL)
           OSPF_EXAMINE_SUMMARIES_ALL (area, rt, rtrs);
     }
