@@ -1,6 +1,6 @@
 /*
  * OSPF inter-area routing.
- * Copyright (C) 1999 Alex Zinin
+ * Copyright (C) 1999, 2000 Alex Zinin, Toshiaki Takada
  *
  * This file is part of GNU Zebra.
  *
@@ -24,6 +24,7 @@
 #define _ZEBRA_OSPF_IA_H
 
 /* Macros. */
+#if 0
 #define OSPF_EXAMINE_SUMMARIES_ALL(A,N,R) \
 	{ \
 	  ospf_examine_summaries ((A), SUMMARY_LSA ((A)), (N), (R)); \
@@ -35,6 +36,19 @@
 	  ospf_examine_transit_summaries ((A), SUMMARY_LSA ((A)), (N), (R)); \
 	  ospf_examine_transit_summaries ((A), SUMMARY_LSA_ASBR ((A)), (N), (R)); \
 	}
+#else
+#define OSPF_EXAMINE_SUMMARIES_ALL(A,N,R) \
+	{ \
+	  ospf_examine_summaries ((A), SUMMARY_LSDB ((A)), (N), (R)); \
+	  ospf_examine_summaries ((A), SUMMARY_ASBR_LSDB ((A)), (N), (R)); \
+	}
+
+#define OSPF_EXAMINE_TRANSIT_SUMMARIES_ALL(A,N,R) \
+	{ \
+	  ospf_examine_transit_summaries ((A), SUMMARY_LSDB ((A)), (N), (R)); \
+	  ospf_examine_transit_summaries ((A), SUMMARY_ASBR_LSDB ((A)), (N), (R)); \
+	}
+#endif
 
 void ospf_ia_routing (struct route_table *, struct route_table *);
 int ospf_area_is_transit (struct ospf_area *);
