@@ -32,13 +32,12 @@ struct vertex
   u_char type;
   struct in_addr id;
   struct lsa_header *lsa;
-  u_int16_t distance;
+  u_int32_t distance;
   list child;
   list nexthop;
-  struct in_addr address; /* Address to use for VLs */
 };
 
-struct ospf_nexthop
+struct vertex_nexthop
 {
   struct interface *ifp;
   struct in_addr router;
@@ -46,4 +45,6 @@ struct ospf_nexthop
 };
 
 void ospf_spf_calculate_schedule ();
-void ospf_spf_calculate_timer_add ();
+void ospf_rtrs_free (struct route_table *);
+
+/* void ospf_spf_calculate_timer_add (); */
