@@ -28,7 +28,6 @@
 #include "memory.h"
 #include "network.h"
 #include "table.h"
-#include "roken.h"
 #include "log.h"
 #include "stream.h"
 #include "thread.h"
@@ -37,6 +36,9 @@
 
 #include "zebra/connected.h"
 
+#ifdef RIP_API
+#include "ripd/ripd_api.h"
+#endif /* RIP_API */
 #include "ripd/ripd.h"
 #include "ripd/rip_debug.h"
 
@@ -108,8 +110,10 @@ rip_interface_new ()
   ri = XMALLOC (MTYPE_RIP_INTERFACE, sizeof (struct rip_interface));
   memset (ri, 0, sizeof (struct rip_interface));
 
-  /* Default authentication type is no auth. */
-  ri->auth_type = RIP_NO_AUTH;
+  /* Default authentication type is simple password for Cisco
+     compatibility. */
+  /* ri->auth_type = RIP_NO_AUTH; */
+  ri->auth_type = RIP_AUTH_SIMPLE_PASSWORD;
 
   /* Set default split-horizon behavior.  If the interface is Frame
      Relay or SMDS is enabled, the default value for split-horizon is
@@ -544,13 +548,16 @@ rip_interface_delete (int command, struct zclient *zclient,
   zlog_info("interface delete %s index %d flags %d metric %d mtu %d",
 	    ifp->name, ifp->ifindex, ifp->flags, ifp->metric, ifp->mtu);  
   
-   if (!IS_IF_PSEUDO(ifp)){
+#ifdef HAVE_IF_PSEUDO
+  if (!IS_IF_PSEUDO(ifp)){
      if_delete(ifp);
    }
    else{
      ifp->ifindex=INTERFACE_PSEUDO;
    }
-
+#else
+  if_delete(ifp);
+#endif /* HAVE_IF_PSEUDO */
   return 0;
 }
 
@@ -566,6 +573,10 @@ rip_interface_clean ()
       ifp = getdata (node);
       ri = ifp->info;
 
+      ri->enable_network = 0;
+      ri->enable_interface = 0;
+      ri->running = 0;
+
       if (ri->t_wakeup)
 	{
 	  thread_cancel (ri->t_wakeup);
@@ -593,7 +604,8 @@ rip_interface_reset ()
       ri->ri_send = RI_RIP_UNSPEC;
       ri->ri_receive = RI_RIP_UNSPEC;
 
-      ri->auth_type = RIP_NO_AUTH;
+      /* ri->auth_type = RIP_NO_AUTH; */
+      ri->auth_type = RIP_AUTH_SIMPLE_PASSWORD;
 
       if (ri->auth_str)
 	{
@@ -750,24 +762,29 @@ int
 rip_interface_address_add (int command, struct zclient *zclient,
 			   zebra_size_t length)
 {
-  struct connected *c;
+  struct connected *ifc;
   struct prefix *p;
 
-  c = zebra_interface_address_add_read (zclient->ibuf);
+  ifc = zebra_interface_address_add_read (zclient->ibuf);
 
-  if (c == NULL)
+  if (ifc == NULL)
     return 0;
 
-  if (IS_RIP_DEBUG_ZEBRA)
-    {
-      p = c->address;
-      if (p->family == AF_INET)
-	zlog_info ("connected address %s/%d", 
-		   inet_ntoa (p->u.prefix4), p->prefixlen);
-    }
+  p = ifc->address;
 
-  /* Check is this interface is RIP enabled or not.*/
-  rip_enable_apply (c->ifp);
+  if (p->family == AF_INET)
+    {
+      if (IS_RIP_DEBUG_ZEBRA)
+	zlog_info ("connected address %s/%d is added", 
+		   inet_ntoa (p->u.prefix4), p->prefixlen);
+      
+      /* Check is this interface is RIP enabled or not.*/
+      rip_enable_apply (ifc->ifp);
+
+#ifdef HAVE_SNMP
+      rip_ifaddr_add (ifc->ifp, ifc);
+#endif /* HAVE_SNMP */
+    }
 
   return 0;
 }
@@ -776,6 +793,30 @@ int
 rip_interface_address_delete (int command, struct zclient *zclient,
 			      zebra_size_t length)
 {
+  struct connected *ifc;
+  struct prefix *p;
+
+  ifc = zebra_interface_address_delete_read (zclient->ibuf);
+  
+  if (ifc)
+    {
+      p = ifc->address;
+      if (p->family == AF_INET)
+	{
+	  if (IS_RIP_DEBUG_ZEBRA)
+
+	    zlog_info ("connected address %s/%d is deleted",
+		       inet_ntoa (p->u.prefix4), p->prefixlen);
+#ifdef HAVE_SNMP
+	  rip_ifaddr_delete (ifc->ifp, ifc);
+#endif /* HAVE_SNMP */
+
+	  /* Check if this interface is RIP enabled or not.*/
+	  rip_enable_apply (ifc->ifp);
+	}
+      connected_free (ifc);
+    }
+
   return 0;
 }
 
@@ -785,6 +826,7 @@ rip_enable_network_lookup (struct interface *ifp)
 {
   listnode listnode;
   struct connected *connected;
+  struct prefix_ipv4 address;
 
   for (listnode = listhead (ifp->connected); listnode; nextnode (listnode))
     if ((connected = getdata (listnode)) != NULL)
@@ -796,7 +838,12 @@ rip_enable_network_lookup (struct interface *ifp)
 
 	if (p->family == AF_INET)
 	  {
-	    node = route_node_match (rip_enable_network, p);
+	    address.family = AF_INET;
+	    address.prefix = p->u.prefix4;
+	    address.prefixlen = IPV4_MAX_BITLEN;
+	    
+	    node = route_node_match (rip_enable_network,
+				     (struct prefix *)&address);
 	    if (node)
 	      {
 		route_unlock_node (node);
@@ -917,6 +964,41 @@ rip_interface_wakeup (struct thread *t)
   return 0;
 }
 
+#ifdef NEW_RIP_TABLE
+void
+rip_interface_route_add (struct interface *ifp)
+{
+  listnode listnode;
+  struct connected *connected;
+  struct prefix *p; 
+  struct route_node *node;
+  struct rip_info *rinfo;
+
+  for (listnode = listhead (ifp->connected); listnode; nextnode (listnode))
+    if ((connected = getdata (listnode)) != NULL)
+      {
+	p = connected->address;
+
+	if (p->family == AF_INET)
+	  {
+	    rinfo = rip_info_new ();
+	    rinfo->type = ZEBRA_ROUTE_CONNECT;
+	    rinfo->sub_type = RIP_ROUTE_INTERFACE;
+	    rinfo->ifindex = ifp->ifindex;
+	    rinfo->metric = 1;
+	    rip_route_add (p, rinfo);
+	    rip_route_process ();
+	  }
+      }
+}
+
+void
+rip_interface_route_delete (struct interface *ifp)
+{
+  ;
+}
+#endif /* NEW_RIP_TABLE */
+
 /* Update interface status. */
 void
 rip_enable_apply (struct interface *ifp)
@@ -961,15 +1043,11 @@ rip_enable_apply (struct interface *ifp)
 	  if (! ri->t_wakeup)
 	    ri->t_wakeup = thread_add_timer (master, rip_interface_wakeup,
 					     ifp, 1);
-#if 0
-	  /* Join to multicast group. */
-	  rip_multicast_join (ifp, rip->sock);
-
-	  /* Send RIP request to the interface. */
-	  rip_request_interface (ifp);
-#endif /* 0 */
-
 	  ri->running = 1;
+
+#ifdef NEW_RIP_TABLE
+	  rip_interface_route_add (ifp);
+#endif /* NEW_RIP_TABLE */
 	}
     }
   else
@@ -983,6 +1061,10 @@ rip_enable_apply (struct interface *ifp)
 	  rip_multicast_leave (ifp, rip->sock);
 
 	  ri->running = 0;
+
+#ifdef NEW_RIP_TABLE
+	  rip_interface_route_delete (ifp);
+#endif /* NEW_RIP_TABLE */
 	}
     }
 }
@@ -1508,7 +1590,8 @@ DEFUN (no_ip_rip_authentication_mode,
   ifp = (struct interface *)vty->index;
   ri = ifp->info;
 
-  ri->auth_type = RIP_NO_AUTH;
+  /* ri->auth_type = RIP_NO_AUTH; */
+  ri->auth_type = RIP_AUTH_SIMPLE_PASSWORD;
 
   return CMD_SUCCESS;
 }
@@ -1541,22 +1624,20 @@ DEFUN (ip_rip_authentication_string,
 
   if (strlen (argv[0]) > 16)
     {
-      vty_out (vty, "RIPv2 authentication string must be shorter than 16%s",
+      vty_out (vty, "%% RIPv2 authentication string must be shorter than 16%s",
 	       VTY_NEWLINE);
       return CMD_WARNING;
     }
 
   if (ri->key_chain)
     {
-      vty_out (vty, "key-chain configuration exists%s", VTY_NEWLINE);
+      vty_out (vty, "%% key-chain configuration exists%s", VTY_NEWLINE);
       return CMD_WARNING;
     }
 
   if (ri->auth_str)
     free (ri->auth_str);
 
-  /* For compatibility before zebra-0.88. */
-  ri->auth_type = RIP_AUTH_SIMPLE_PASSWORD;
   ri->auth_str = strdup (argv[0]);
 
   return CMD_SUCCESS;
@@ -1581,7 +1662,6 @@ DEFUN (no_ip_rip_authentication_string,
   if (ri->auth_str)
     free (ri->auth_str);
 
-  /* ri->auth_type = RIP_NO_AUTH; */
   ri->auth_str = NULL;
 
   return CMD_SUCCESS;
@@ -1611,6 +1691,13 @@ DEFUN (ip_rip_authentication_key_chain,
   ifp = (struct interface *) vty->index;
   ri = ifp->info;
 
+  if (ri->auth_str)
+    {
+      vty_out (vty, "%% authentication string configuration exists%s",
+	       VTY_NEWLINE);
+      return CMD_WARNING;
+    }
+
   if (ri->key_chain)
     free (ri->key_chain);
 
@@ -1724,9 +1811,11 @@ rip_interface_config_write (struct vty *vty)
 	vty_out (vty, " description %s%s", ifp->desc,
 		 VTY_NEWLINE);
 
-       if (IS_IF_PSEUDO (ifp))
+#ifdef HAVE_IF_PSEUDO
+      if (IS_IF_PSEUDO (ifp))
 	 vty_out (vty, " pseudo %s", VTY_NEWLINE);
-
+#endif /* HAVE_IF_PSEUDO */
+      
       /* Split horizon. */
       if (ri->split_horizon != ri->split_horizon_default)
 	{
@@ -1748,8 +1837,11 @@ rip_interface_config_write (struct vty *vty)
 		 VTY_NEWLINE);
 
       /* RIP authentication. */
+#if 0 
+      /* RIP_AUTH_SIMPLE_PASSWORD becomes default mode. */
       if (ri->auth_type == RIP_AUTH_SIMPLE_PASSWORD)
 	vty_out (vty, " ip rip authentication mode text%s", VTY_NEWLINE);
+#endif /* 0 */
       if (ri->auth_type == RIP_AUTH_MD5)
 	vty_out (vty, " ip rip authentication mode md5%s", VTY_NEWLINE);
 
@@ -1811,6 +1903,7 @@ struct cmd_node interface_node =
 {
   INTERFACE_NODE,
   "%s(config-if)# ",
+  1,
 };
 
 /* Called when interface structure allocated. */
@@ -1853,9 +1946,10 @@ rip_if_init ()
   install_default (INTERFACE_NODE);
   install_element (INTERFACE_NODE, &interface_desc_cmd);
   install_element (INTERFACE_NODE, &no_interface_desc_cmd);
+#ifdef HAVE_IF_PSEUDO  
   install_element (INTERFACE_NODE, &interface_pseudo_cmd);
   install_element (INTERFACE_NODE, &no_interface_pseudo_cmd);
-
+#endif /* HAVE_IF_PSEUDO */
   install_element (RIP_NODE, &rip_network_cmd);
   install_element (RIP_NODE, &no_rip_network_cmd);
   install_element (RIP_NODE, &rip_neighbor_cmd);
@@ -1889,3 +1983,437 @@ rip_if_init ()
   install_element (INTERFACE_NODE, &rip_split_horizon_cmd);
   install_element (INTERFACE_NODE, &no_rip_split_horizon_cmd);
 }
+
+#ifdef RIP_API
+/****** API ****/
+
+/*************************
+ Function:ripd_api_enable_ip_if
+ PURPOSE: Enable RIP network
+ PARAMETERS:
+    IN    struct prefix* prefx : IP prefix
+    IN    u_char enable_it : 0 - disable, else - enable
+**************************/
+int
+ripd_api_enable_ip_if (struct prefix* prefx, u_char enable_it)
+{
+  int ret;
+
+  if (enable_it)
+    ret = rip_enable_network_add (prefx);
+  else
+    ret = rip_enable_network_delete (prefx);
+
+  if (ret < 0)
+    {
+      return CMD_WARNING;
+    }
+
+  rip_enable_apply_all ();
+
+  return CMD_SUCCESS;
+}
+
+/*************************
+ Function:ripd_api_enable_network
+ PURPOSE: Enable RIP network
+ PARAMETERS:
+    IN    char* if_name : IP prefix or interface name
+    IN    u_char enable_it : 0 - disable, else - enable
+**************************/
+int
+ripd_api_enable_network (char* if_name, u_char enable_it)
+{
+  int ret;
+  prefix_ipv4_t p;
+
+  ret = str2prefix_ipv4 (if_name, &p);
+  if (ret)
+    return ripd_api_enable_ip_if ((struct prefix *) &p, enable_it);
+
+  if (enable_it)
+    {
+      ret = rip_enable_if_add (if_name);
+    }
+  else
+    {
+      ret = rip_enable_if_delete (if_name);
+    }
+
+  if (ret < 0)
+    {
+      return CMD_WARNING;
+    }
+
+  rip_enable_apply_all ();
+
+  return CMD_SUCCESS;
+}
+
+/*************************
+ Function:ripd_api_enable_neighbor
+ PURPOSE: Enable/Disable RIP neighbor router
+ PARAMETERS:
+    IN    struct prefix* prefx : IP prefix
+    IN    u_char enable_it : 0 - disable, else - enable
+**************************/
+int
+ripd_api_enable_neighbor (prefix_ipv4_t* prefx, u_char enable_it)
+{
+  if (enable_it)
+    rip_neighbor_add (prefx);
+  else
+    rip_neighbor_delete (prefx);
+
+  return CMD_SUCCESS;
+}
+
+/*************************
+ Function:ripd_api_get_if_running
+ PURPOSE: Return the flag : is RIP running on this interface.
+ PARAMETERS:
+    IN    interface_t* ifp - inteface to be checked
+    OUT   int *running - pointer to result
+**************************/
+int
+ripd_api_get_if_running (interface_t *ifp, int *running)
+{
+  rip_interface_t *ri;
+
+  ri = ifp->info;
+  *running = ri->running;
+  return CMD_SUCCESS;
+}
+
+/*************************
+ Function:ripd_api_set_if_rx_version
+ PURPOSE: Set interface's receive RIP version control
+ PARAMETERS:
+    IN    interface_t* ifp - inteface to be configured
+    IN    int ri_version - RIP accepet/announce method
+**************************/
+int
+ripd_api_set_if_rx_version (interface_t *ifp, int ri_version)
+{
+  rip_interface_t *ri;
+
+  ri = ifp->info;
+  ri->ri_receive = ri_version;
+  return CMD_SUCCESS;
+}
+
+/*************************
+ Function:ripd_api_get_if_rx_version
+ PURPOSE: Get interface's receive RIP version control
+ PARAMETERS:
+    IN    interface_t* ifp - inteface to be configured
+    OUT   int* ri_version - pointer for result
+**************************/
+int
+ripd_api_get_if_rx_version (interface_t *ifp, int *ri_version)
+{
+  rip_interface_t *ri;
+
+  ri = ifp->info;
+
+  if (! ri->running || !rip || !(ri->enable_network || ri->enable_interface))
+    {
+      *ri_version = 0;
+      return CMD_WARNING;
+    }
+
+  if (ri->ri_receive == RI_RIP_UNSPEC)
+    *ri_version=rip->version;
+  else
+    *ri_version=ri->ri_receive;
+    
+  return CMD_SUCCESS;
+}
+
+/*************************
+ Function:ripd_api_set_if_tx_version
+ PURPOSE: Set interface's send RIP version control
+ PARAMETERS:
+    IN    interface_t* ifp - inteface to be configured
+    IN    int ri_version - RIP accepet/announce method
+**************************/
+int
+ripd_api_set_if_tx_version (interface_t *ifp, int ri_version)
+{
+  rip_interface_t *ri;
+
+  ri = ifp->info;
+  ri->ri_send = ri_version;
+
+  return CMD_SUCCESS;
+}
+
+/*************************
+ Function:ripd_api_get_if_tx_version
+ PURPOSE: Get interface's send RIP version control
+ PARAMETERS:
+    IN    interface_t* ifp - inteface to be configured
+    OUT   int* ri_version - pointer for result
+**************************/
+int
+ripd_api_get_if_tx_version (interface_t *ifp, int *ri_version)
+{
+  rip_interface_t *ri;
+
+  ri = ifp->info;
+
+  if (! ri->running || !rip || !(ri->enable_network || ri->enable_interface)){    
+    *ri_version = 0;
+    return CMD_WARNING;
+  }
+
+  if (ri->ri_send == RI_RIP_UNSPEC)
+    *ri_version=rip->version;
+  else
+    *ri_version=ri->ri_send;
+    
+  return CMD_SUCCESS;
+}
+
+/*************************
+ Function:ripd_api_set_if_authentication_type
+ PURPOSE: Set RIP authentication type
+ PARAMETERS:
+    IN    interface_t* ifp - inteface to be configured
+    IN    int  auth_type - RIPv2 authentication type
+                            (from {RIP_NO_AUTH,
+                                   RIP_AUTH_SIMPLE_PASSWORD,
+                                   RIP_AUTH_MD5})
+**************************/
+int
+ripd_api_set_if_authentication_type (interface_t *ifp, int auth_type)
+{
+  rip_interface_t *ri;
+
+  if (RIP_NO_AUTH != auth_type &&
+      RIP_AUTH_SIMPLE_PASSWORD != auth_type &&
+      RIP_AUTH_MD5 != auth_type)
+    {
+      fprintf (stderr, "invalid auth.type %d\n", auth_type);
+      return CMD_ERR_AMBIGUOUS;
+    }
+
+  ri = ifp->info;
+
+  ri->auth_type = auth_type;
+
+  return CMD_SUCCESS;
+}
+
+/*************************
+ Function:ripd_api_set_if_authentication_string
+ PURPOSE: RIP authentication string setting
+ PARAMETERS:
+    IN    interface_t* ifp - inteface to be configured
+    IN    char* auth_str - RIP authentication string;
+                           if auth_str==NULL, authentication string is disabled
+**************************/
+int
+ripd_api_set_if_authentication_string (interface_t* ifp, char* auth_str)
+{
+  rip_interface_t *ri;
+
+  ri = ifp->info;
+
+  if (auth_str && *auth_str)
+    {
+      if (strlen (auth_str) > 16)
+        {
+          zlog_err ("api_set:auth_str is too long (%d)", (int) strlen (auth_str));
+          return CMD_WARNING;
+        }
+
+      if (ri->auth_str)
+        free (ri->auth_str);
+
+      ri->auth_str = strdup (auth_str);
+    }
+  else
+    {
+      if (ri->auth_str)
+        free (ri->auth_str);
+
+      ri->auth_type = RIP_NO_AUTH;
+      ri->auth_str = NULL;
+    }
+
+  return CMD_SUCCESS;
+}
+
+/*************************
+ Function:ripd_api_get_if_authentication_string
+ PURPOSE: GET RIP authentication string
+ PARAMETERS:
+    IN    interface_t* ifp - inteface to be configured
+    IN    int buffer_size - size of the buffer for the result
+    OUT   char* auth_str - pointer to buffer for result (actual length
+                           may be counted as its 'strlen')
+    OUT   int*  auth_type - RIPv2 authentication type
+                            (from {RIP_NO_AUTH,
+                                   RIP_AUTH_SIMPLE_PASSWORD,
+                                   RIP_AUTH_MD5})
+**************************/
+int
+ripd_api_get_if_authentication_string (interface_t* ifp,
+                                       int buffer_size,
+                                       char* auth_str,
+                                       int* auth_type)
+{
+  rip_interface_t *ri;
+
+  ri = ifp->info;
+
+  if (! ri)
+    return CMD_WARNING;
+
+  if (auth_type)
+    *auth_type = ri->auth_type;
+
+  if (! auth_str)
+    return CMD_SUCCESS;
+
+  if (ri->auth_str)
+    {
+      strncpy (auth_str, ri->auth_str, buffer_size);
+      if (strlen (ri->auth_str) >= buffer_size)
+        { /* it is too long => cut it ! */
+          zlog_err ("api_get:auth_str is too long, cut it");
+          auth_str[buffer_size - 1] = '\0';
+          return CMD_WARNING; /* the buffer has been cut */
+        }
+    }
+  else
+    {/* build the empty string */
+      //zlog_warn ("api_get:auth_str is empty");
+      auth_str[0] = '\0';
+      if (RIP_NO_AUTH != ri->auth_type)
+        return CMD_WARNING; /* unconsistent */
+    }
+
+  return CMD_SUCCESS;
+}
+
+/*************************
+ Function:ripd_api_set_if_split_horizon
+ PURPOSE: Set interface's send RIP version control
+ PARAMETERS:
+    IN    interface_t* ifp - inteface to be configured
+    IN    int new_split_mode_value
+**************************/
+int
+ripd_api_set_if_split_horizon (interface_t *ifp, int new_split_mode_value)
+{
+  rip_interface_t *ri;
+
+  ri = ifp->info;
+  ri->split_horizon = new_split_mode_value;
+
+  return CMD_SUCCESS;
+}
+
+/*************************
+ Function:ripd_api_get_if_split_horizon
+ PURPOSE: Set interface's send RIP version control
+ PARAMETERS:
+    IN    interface_t* ifp - inteface to be configured
+    OUT   int* current_split_mode_value
+**************************/
+int
+ripd_api_get_if_split_horizon (interface_t *ifp, int* current_split_mode_value)
+{
+  rip_interface_t *ri;
+
+  ri = ifp->info;
+  *current_split_mode_value = ri->split_horizon;
+
+  return CMD_SUCCESS;
+}
+
+/*************************
+ Function:ripd_api_set_if_description
+ PURPOSE: Set/Delete interface description
+ PARAMETERS:
+    IN    interface_t* ifp - inteface to be configured
+    IN    char *description (if NULL => delete)
+**************************/
+int
+ripd_api_set_if_description (interface_t *ifp, char *description)
+{
+  if (ifp->desc) /* anyway, delete old one */
+    XFREE (0, ifp->desc);
+
+  if (description && *description)
+    { /* set non-empty description */
+      ifp->desc = XMALLOC (MTYPE_BUFFER,  1 + strlen (description));
+      strcpy (ifp->desc, description);
+    }
+  else
+    { /* delete description */
+      ifp->desc = NULL;
+    }
+
+  return CMD_SUCCESS;
+}
+
+/*************************
+ Function:ripd_api_get_if_description
+ PURPOSE: Get interface description
+ PARAMETERS:
+    IN    interface_t* ifp - inteface to be configured
+    IN    int buffer_size
+    OUT   char *description
+**************************/
+int
+ripd_api_get_if_description (interface_t *ifp, int buffer_size,
+			     char *description)
+{
+  if (! description || buffer_size < 2)
+    return CMD_WARNING; /* unconsistent */
+
+  if (ifp->desc)
+    {
+      strncpy (description, ifp->desc, buffer_size);
+      description[buffer_size - 1] = '\0';
+    }
+
+  return CMD_SUCCESS;
+}
+
+/*************************
+ Function:ipd_api_get_if_statistics
+ PURPOSE: Get RIPD interface statistics
+ PARAMETERS:
+    IN    interface_t* ifp - inteface to be configured
+    OUT   u_long *recv_badpackets
+    OUT   u_long *recv_badroutes
+    OUT   u_long *sent_updates
+**************************/
+int
+ripd_api_get_if_statistics (interface_t *ifp,
+                            u_long *recv_badpackets,
+                            u_long *recv_badroutes,
+                            u_long *sent_updates)
+{
+  rip_interface_t *ri;
+
+  ri = ifp->info;
+  if (! ri)
+    return CMD_WARNING; /* unconsistent */
+
+  if (recv_badpackets)
+    *recv_badpackets = ri->recv_badpackets;
+
+  if (recv_badroutes)
+    *recv_badroutes = ri->recv_badroutes;
+
+  if (sent_updates)
+    *sent_updates = ri->sent_updates;
+
+  return CMD_SUCCESS;
+}
+#endif /* RIP_API */
